@@ -1,5 +1,5 @@
 /*
- * "$Id: print-util.c,v 1.115 2007/09/02 18:11:51 rlk Exp $"
+ * "$Id: print-util.c,v 1.117 2010/06/26 20:02:02 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -124,6 +124,12 @@ stp_zfwrite(const char *buf, size_t bytes, size_t nitems, const stp_vars_t *v)
   (stp_get_outfunc(v))((void *)(stp_get_outdata(v)), buf, bytes * nitems);
 }
 
+void
+stp_write_raw(const stp_raw_t *raw, const stp_vars_t *v)
+{
+  (stp_get_outfunc(v))((void *)(stp_get_outdata(v)), raw->data, raw->bytes);
+}
+
 void
 stp_putc(int ch, const stp_vars_t *v)
 {
@@ -541,7 +547,7 @@ stp_strdup(const char *s)
 const char *
 stp_set_output_codeset(const char *codeset)
 {
-#ifdef ENABLE_NLS
+#if defined(ENABLE_NLS) && !defined(__APPLE__)
   return (const char *)(bind_textdomain_codeset(PACKAGE, codeset));
 #else
   return "US-ASCII";
